"""
    Apideck

    The Apideck OpenAPI Spec: SDK Optimized  # noqa: E501

    The version of the OpenAPI document: 10.8.3
    Contact: support@apideck.com
    Generated by: https://openapi-generator.tech
"""


import sys
import unittest

import apideck
from apideck.model.allocation import Allocation
from apideck.model.currency import Currency
from apideck.model.custom_field import CustomField
from apideck.model.deprecated_linked_supplier import DeprecatedLinkedSupplier
from apideck.model.linked_customer import LinkedCustomer
from apideck.model.linked_ledger_account import LinkedLedgerAccount
from apideck.model.linked_tracking_categories import LinkedTrackingCategories
from apideck.model.pass_through_body import PassThroughBody
from apideck.model.payment_status import PaymentStatus
from apideck.model.payment_type import PaymentType
globals()['Allocation'] = Allocation
globals()['Currency'] = Currency
globals()['CustomField'] = CustomField
globals()['DeprecatedLinkedSupplier'] = DeprecatedLinkedSupplier
globals()['LinkedCustomer'] = LinkedCustomer
globals()['LinkedLedgerAccount'] = LinkedLedgerAccount
globals()['LinkedTrackingCategories'] = LinkedTrackingCategories
globals()['PassThroughBody'] = PassThroughBody
globals()['PaymentStatus'] = PaymentStatus
globals()['PaymentType'] = PaymentType
from apideck.model.payment import Payment


class TestPayment(unittest.TestCase):
    """Payment unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testPayment(self):
        """Test Payment"""
        # FIXME: construct object with mandatory attributes with example values
        # model = Payment()  # noqa: E501
        pass


if __name__ == '__main__':
    unittest.main()
